@@ -1,5 +1,5 @@
 package App::Wallflower;
-$App::Wallflower::VERSION = '1.009';
+$App::Wallflower::VERSION = '1.010';
 use strict;
 use warnings;
 
@@ -10,6 +10,7 @@ use Plack::Util ();
 use URI;
 use Wallflower;
 use Wallflower::Util qw( links_from );
+use List::Util qw( uniqstr );
 
 sub _default_options {
     return (
@@ -71,8 +72,9 @@ sub new_with_options {
         'verbose!',      'errors!',                 'tap!',
         'host=s@',
         'url|uri=s',
+        'parallel=i',
         'help',          'manual',
-        'tutorial',
+        'tutorial',      'version',
     ) or pod2usage(
         -input   => $input,
         -verbose => 1,
@@ -92,6 +94,8 @@ sub new_with_options {
             Pod::Find::pod_where( { -inc => 1 }, 'Wallflower::Tutorial' );
         },
     ) if $option{tutorial};
+    print "wallflower version $Wallflower::VERSION\n" and exit
+      if $option{version};
 
     # application is required
     pod2usage(
@@ -121,6 +125,11 @@ sub new {
     if ( $option{tap} ) {
         require Test::More;
         import Test::More;
+        if ( $option{parallel} ) {
+            my $tb = Test::Builder->new;
+            $tb->no_plan;
+            $tb->use_numbers(0);
+        }
         $option{quiet} = 1;    # --tap = --quiet
         if ( !exists $option{destination} ) {
             require File::Temp;
@@ -145,11 +154,11 @@ sub new {
 
     local $ENV{PLACK_ENV} = $option{environment};
     local @INC = ( @{ $option{inc} }, @INC );
-    return bless {
+    my $self = {
         option     => \%option,
         args       => $args,
         callbacks  => \@cb,
-        seen       => {},
+        seen       => {},                # keyed on $url->path
         wallflower => Wallflower->new(
             application => ref $option{application}
                 ? $option{application}
@@ -158,8 +167,19 @@ sub new {
             ( index       => $option{index}       )x!! $option{index},
             ( url         => $option{url}         )x!! $option{url},
         ),
-    }, $class;
+    };
 
+    # setup parallel processing
+    if ( $self->{option}{parallel} ) {
+        require Path::Tiny;
+        require Fcntl;
+        import Fcntl qw( :seek :flock );
+        $self->{_parent_}  = $$;
+        $self->{_forked_}  = 0;
+        $self->{_ipc_dir_} = Path::Tiny->tempdir( CLEANUP => 1 );
+    }
+
+    return bless $self, $class;
 }
 
 sub run {
@@ -167,7 +187,87 @@ sub run {
     ( my $args, $self->{args} ) = ( $self->{args}, [] );
     my $method = $self->{option}{filter} ? '_process_args' : '_process_queue';
     $self->$method(@$args);
-    done_testing() if $self->{option}{tap};
+    if    ( $self->{option}{parallel} ) { $self->_wait_for_kids; }
+    elsif ( $self->{option}{tap} )      { done_testing(); }
+}
+
+sub __open_fh {
+    my ($file) = @_;
+    open my $fh, -e $file ? '+<' : '+>', $file
+      or die "Can't open $file in read-write mode: $!";
+    $fh->autoflush(1);
+    $fh;
+}
+
+sub _has_seen {
+    my ( $self, $path, $queue ) = @_;
+
+    if ( $self->{option}{parallel} ) {
+
+        $self->_update_seen($path);
+
+        # clean the queue after the update
+        @$queue = uniqstr grep !ref || !$self->{seen}{$_->path }, @$queue;
+
+        # fork new kids if necessary
+        if (   @$queue
+            && $self->{_parent_} == $$
+            && $self->{_forked_} < $self->{option}{parallel} - 1 )
+        {
+            my @seed = splice @$queue, 0, @$queue / 2;
+            if ( not my $pid = fork ) {
+                $self->{_pidfile_} = File::Temp->new(
+                    TEMPLATE => "pid-$$-XXXX",
+                    DIR      => $self->{_ipc_dir_}
+                );
+                @$queue = @seed;
+                delete $self->{_seen_fh_};    # will reopen
+            }
+            elsif ( !defined $pid ) {
+                warn "Couldn't fork: $!";
+                unshift @$queue, @seed;
+            }
+            else {
+                $self->{_forked_}++;
+                return ++$self->{seen}{$path}; # $path will be processed by the child
+            }
+        }
+    }
+
+    return $self->{seen}{$path}++;
+}
+
+sub _update_seen {
+    my ( $self, @seen ) = @_;
+    my $seen = $self->{seen};
+
+    # read from the shared seen file
+    my $SEEN = $self->{_ipc_dir_}->child('__SEEN__');
+    my $fh = $self->{_seen_fh_} ||= __open_fh($SEEN);
+    flock( $fh, LOCK_EX() ) or die "Cannot lock $SEEN: $!\n";
+    seek( $fh, 0, SEEK_CUR() );
+    while (<$fh>) { chomp; $seen->{$_}++; }
+
+    # write to the shared seen file
+    for my $path (@seen) {
+        seek( $fh, 0, SEEK_END() );
+        print $fh "$path\n" if !$seen->{$path};
+    }
+    flock( $fh, LOCK_UN() ) or die "Cannot unlock $SEEN: $!\n";
+}
+
+sub _wait_for_kids {
+    my ($self) = @_;
+    return if $self->{_parent_} != $$;
+    sleep 1 while @{ [ glob( $self->{_ipc_dir_}->child('pid-*') ) ] };
+    if( $self->{option}{tap} ) {
+        seek my $fh = $self->{_seen_fh_}, 0, SEEK_SET();
+        my $count;
+        $count++ while <$fh>;
+        my $tb = Test::Builder->new;
+        $tb->no_ending(1);
+        $tb->done_testing($count);
+    }
 }
 
 sub _process_args {
@@ -197,7 +297,7 @@ sub _process_queue {
     while (@queue) {
 
         my $url = URI->new( shift @queue );
-        next if $seen->{ $url->path }++;
+        next if $self->_has_seen( $url->path, \@queue );
         next if $url->scheme && ! eval { $url->host =~ $host_ok };
 
         # get the response
@@ -209,7 +309,10 @@ sub _process_queue {
         # obtain links to resources
         my ( $status, $headers, $file ) = @$response;
         if ( $status eq '200' && $follow ) {
-            push @queue, links_from( $response => $url );
+            @queue = uniqstr
+              grep !ref || !$seen->{$_->path},
+              @queue,
+             links_from( $response => $url );
         }
 
         # follow 301 Moved Permanently
@@ -241,7 +344,7 @@ App::Wallflower - Class performing the moves for the wallflower program
 
 =head1 VERSION
 
-version 1.009
+version 1.010
 
 =head1 SYNOPSIS
 
